package com.contextspace.internalexception;

import javax.jdo.annotations.Index;
import javax.jdo.annotations.Indices;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Table;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Entity
@Table
@Indices({
	@Index(members={"party", "relationshipType"}),
	@Index(members={"otherParty", "relationshipType"})
    //@Index(members={"relationshipType"}) - do not use this again, it causes performance issues
})
public class Relationship extends BaseObject { //implements Comparable<Relationship> {
    private static final long serialVersionUID = 1L;

    static Logger log = LoggerFactory.getLogger(Relationship.class.getName());
    
    public static enum RelationshipType {
        PARENT, OWNER, ADMIN, CONTAINS, MEMBER, EMPLOYEE, MANAGER
    }
    
    private BaseObject party;
    
    private BaseObject otherParty;
    
    private RelationshipType relationshipType;
    
    
    /**
     * Default Constructor
     */
    public Relationship() {
        super();
    }
    
    /**
     * Constructor taking the two parties and the relationship between them
     * @param party
     * @param otherParty
     * @param type
     */
    public Relationship(BaseObject party, BaseObject otherParty, RelationshipType type) {
        super();
        this.party = party;
        this.otherParty = otherParty;
        this.relationshipType = type;
        
    }
    
    /**
     * @return the party
     */
    public BaseObject getParty() {
        return party;
    }
    /**
     * @param party the party to set
     */
    public void setParty(BaseObject party) {
        this.party = party;
    }
    /**
     * @return the otherParty
     */
    public BaseObject getOtherParty() {
        return otherParty;
    }
    /**
     * @param otherParty the otherParty to set
     */
    public void setOtherParty(BaseObject otherParty) {
        this.otherParty = otherParty;
    }
    
    /**
     * @return the relationshipType
     */
    @Enumerated(EnumType.STRING)
    public RelationshipType getRelationshipType() {
        return relationshipType;
    }
    /**
     * @param relationshipType the relationshipType to set
     */
    public void setRelationshipType(RelationshipType relationshipType) {
        this.relationshipType = relationshipType;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("<relationship type=\"").append(relationshipType).append("\"");
        if(getId() != 0) {
            buf.append(" id=\"").append(getId()).append("\"");
        }
        buf.append(">");
        if(party != null) {
            buf.append("<party class=\"").append(party.getClass().getName()).append("\" id=\"").append(party.getId()).append("\"/>");
        }
        if(otherParty != null) {
            buf.append("<otherParty class=\"").append(otherParty.getClass().getName()).append("\" id=\"").append(otherParty.getId()).append("\"/>");
        }
        buf.append(super.toString());
        buf.append("</relationship>");
        
        return buf.toString();
    }


    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if(obj == null || !(obj instanceof Relationship)) {
            return false;
        }
        Relationship otherRelationship = (Relationship)obj;
        
        if(getId()!=0 && otherRelationship.getId() != 0) {
            if(getId() == otherRelationship.getId()) {
                return true;
            }
            return false;
        }
        else {
            //one of the ids or both isn't set, so we can't tell by the ids
            //whether these are the same objects, so look at the properties.
            if(getParty().equals(otherRelationship.getParty()) && getOtherParty().equals(otherRelationship.getOtherParty())) {
                log.debug("Relationships same. Matched on parties, as ids not set (maybe not persisted to db yet)");
                return true;
            }
            log.debug("Relationships NOT the same. Matched on parties, as ids not set (maybe not persisted to db yet)");
            return false;
        }
    }
}
